@@ -81,10 +81,12 @@ echo_log() {
 random_01() {
     set +e
     eval random_01_$1="`expr $RANDOM % 2`"
+    eval echo "random_01_$1=\$random_01_$1"
     set -e
 }
 # -----------------------------------------------------------------------------
 # start new check_all.log
+echo "date > check_all.log"
 date > check_all.log
 top_srcdir=`pwd`
 echo "top_srcdir = $top_srcdir"
@@ -165,9 +167,8 @@ fi
 # Run automated checks for the form bin/check_*.sh with a few exceptions.
 list=`ls bin/check_* | sed \
     -e '/check_all.sh/d' \
-    -e '/check_doxygen.sh/d' \
     -e '/check_jenkins.sh/d' \
-    -e '/check_svn_dist.sh/d'`
+    -e '/check_doxygen.sh/d'`
 # ~/devel/check_copyright.sh not included in batch_edit branch
 for check in $list
 do
@@ -186,7 +187,9 @@ echo_log_eval cd cppad-$version
 echo_log_eval bin/run_cmake.sh $compiler $standard $debug_which $package_vector
 echo_log_eval cd build
 # -----------------------------------------------------------------------------
-echo_log_eval make check
+# can comment out this make check to if only running tests below it
+n_job=`nproc`
+echo_log_eval make -j $n_job check
 # -----------------------------------------------------------------------------
 skip=''
 for package in adolc eigen ipopt fadbad sacado
@@ -200,14 +203,21 @@ do
         skip="$skip $package"
     fi
 done
-#
+# ----------------------------------------------------------------------------
 # extra speed tests not run with option specified
+#
+# make speed_cppad incase make check above is commented out
+echo_log_eval make -j $n_job speed_cppad
 for option in onetape colpack optimize atomic memory boolsparsity
 do
+    #
     echo_eval speed/cppad/speed_cppad correct 432 $option
 done
 if ! echo "$skip" | grep 'adolc' > /dev/null
 then
+    # make speed_adolc incase make check above is commented out
+    echo_log_eval make -j $n_job speed_adolc
+    #
     echo_eval speed/adolc/speed_adolc correct         432 onetape
     echo_eval speed/adolc/speed_adolc sparse_jacobian 432 onetape colpack
     echo_eval speed/adolc/speed_adolc sparse_hessian  432 onetape colpack
@@ -218,61 +228,63 @@ fi
 program_list=''
 for threading in bthread openmp pthread
 do
-    program="example/multi_thread/${threading}"
-    program="$program/example_multi_thread_${threading}"
-    if [ ! -e $program ]
+    dir="example/multi_thread/$threading"
+    if [ ! -e "$dir" ]
     then
-        skip="$skip $program"
+        skip="$skip example_multi_thread_${threading}"
     else
+        program="$dir/example_multi_thread_${threading}"
         program_list="$program_list $program"
         #
-        # fast cases, test for all programs
-        echo_log_eval ./$program a11c
-        echo_log_eval ./$program simple_ad
-        echo_log_eval ./$program team_example
+        # make program incase make check above is commented out
+        echo_log_eval make -j $n_job example_multi_thread_${threading}
+        #
+        # all programs check the fast cases
+        echo_log_eval $program a11c
+        echo_log_eval $program simple_ad
+        echo_log_eval $program team_example
     fi
 done
 if [ "$program_list" != '' ]
 then
     # test_time=1,max_thread=4,mega_sum=1
     next_program
-    echo_log_eval ./$program harmonic 1 4 1
+    echo_log_eval $program harmonic 1 4 1
     #
     # test_time=1,max_thread=4,num_solve=100
     next_program
-    echo_log_eval ./$program multi_atomic 1 4 100
-    #
-    # test_time=1,max_thread=4,num_solve=100
+    echo_log_eval $program atomic_two 1 4 100
     next_program
-    echo_log_eval ./$program checkpoint 1 4 100
+    echo_log_eval $program atomic_three 1 4 100
+    next_program
+    echo_log_eval $program chkpoint_one 1 4 100
+    next_program
+    echo_log_eval $program chkpoint_two 1 4 100
     #
     # test_time=2,max_thread=4,num_zero=20,num_sub=30,num_sum=50,use_ad=true
     next_program
-    echo_log_eval ./$program multi_newton 2 4 20 30 50 true
-    #
+    echo_log_eval $program multi_newton 2 4 20 30 50 true
 fi
 #
 # print_for test
 program='example/print_for/example_print_for'
-if [ ! -e "$program" ]
+# make program incase make check above is commented out
+echo_log_eval make -j $n_job example_print_for
+echo_log_eval $program
+$program | sed -e '/^Test passes/,$d' > junk.1.$$
+$program | sed -e '1,/^Test passes/d' > junk.2.$$
+if diff junk.1.$$ junk.2.$$
 then
-    skip="$skip $program"
+    rm junk.1.$$ junk.2.$$
+    echo_log_eval echo "print_for: OK"
 else
-    echo_log_eval $program
-    $program | sed -e '/^Test passes/,$d' > junk.1.$$
-    $program | sed -e '1,/^Test passes/d' > junk.2.$$
-    if diff junk.1.$$ junk.2.$$
-    then
-        rm junk.1.$$ junk.2.$$
-        echo_log_eval echo "print_for: OK"
-    else
-        echo_log_eval echo "print_for: Error"
-        exit 1
-    fi
+    echo_log_eval echo "print_for: Error"
+    exit 1
 fi
 #
 echo_log_eval make install
 #
+echo "date >> check_all.log"
 date >> $top_srcdir/check_all.log
 if [ "$skip" != '' ]
 then
