@@ -1,7 +1,7 @@
 # ifndef CPPAD_LOCAL_SWEEP_FOR_JAC_HPP
 # define CPPAD_LOCAL_SWEEP_FOR_JAC_HPP
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-18 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-19 Bradley M. Bell
 
 CppAD is distributed under the terms of the
              Eclipse Public License Version 2.0.
@@ -41,7 +41,7 @@ this operation sequence was recorded using AD<Base>.
 
 \tparam Vector_set
 is the type used for vectors of sets. It can be either
-sparse_pack or sparse_list.
+sparse::pack_setvec or sparse::list_setvec.
 
 \param dependency
 Are the derivatives with respect to left and right of the expression below
@@ -148,9 +148,8 @@ void for_jac(
     //
     // pointer to the beginning of the parameter vector
     // (used by atomic functions)
-    const Base* parameter = CPPAD_NULL;
-    if( num_par > 0 )
-        parameter = play->GetPar();
+    CPPAD_ASSERT_UNKNOWN( num_par > 0 )
+    const Base* parameter = play->GetPar();
     //
     // which parametes are dynamic
     const pod_vector<bool>& dyn_par_is( play->dyn_par_is() );
@@ -182,7 +181,7 @@ void for_jac(
         {
             case AbsOp:
             CPPAD_ASSERT_NARG_NRES(op, 1, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -190,7 +189,7 @@ void for_jac(
 
             case AddvvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_binary_op(
+            sparse::for_jac_binary_op(
                 i_var, arg, var_sparsity
             );
             break;
@@ -198,7 +197,7 @@ void for_jac(
 
             case AddpvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[1]), var_sparsity
             );
             break;
@@ -207,7 +206,7 @@ void for_jac(
             case AcosOp:
             // sqrt(1 - x * x), acos(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -217,7 +216,7 @@ void for_jac(
             case AcoshOp:
             // sqrt(x * x - 1), acosh(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -227,7 +226,7 @@ void for_jac(
             case AsinOp:
             // sqrt(1 - x * x), asin(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -237,7 +236,7 @@ void for_jac(
             case AsinhOp:
             // sqrt(1 + x * x), asinh(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -247,7 +246,7 @@ void for_jac(
             case AtanOp:
             // 1 + x * x, atan(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -257,7 +256,7 @@ void for_jac(
             case AtanhOp:
             // 1 - x * x, atanh(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -287,7 +286,7 @@ void for_jac(
             case CosOp:
             // sin(x), cos(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -296,7 +295,7 @@ void for_jac(
             case CoshOp:
             // sinh(x), cosh(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -305,7 +304,7 @@ void for_jac(
             case DisOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
             // derivative is identically zero but dependency is not
-            if( dependency ) forward_sparse_jacobian_unary_op(
+            if( dependency ) sparse::for_jac_unary_op(
                 i_var, size_t(arg[1]), var_sparsity
             );
             else
@@ -315,7 +314,7 @@ void for_jac(
 
             case DivvvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_binary_op(
+            sparse::for_jac_binary_op(
                 i_var, arg, var_sparsity
             );
             break;
@@ -323,7 +322,7 @@ void for_jac(
 
             case DivpvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[1]), var_sparsity
             );
             break;
@@ -331,7 +330,7 @@ void for_jac(
 
             case DivvpOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -344,10 +343,11 @@ void for_jac(
             // -------------------------------------------------
 
             case ErfOp:
+            case ErfcOp:
             // arg[1] is always the parameter 0
             // arg[0] is always the parameter 2 / sqrt(pi)
             CPPAD_ASSERT_NARG_NRES(op, 3, 5);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -355,7 +355,7 @@ void for_jac(
 
             case ExpOp:
             CPPAD_ASSERT_NARG_NRES(op, 1, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -364,7 +364,7 @@ void for_jac(
 # if CPPAD_USE_CPLUSPLUS_2011
             case Expm1Op:
             CPPAD_ASSERT_NARG_NRES(op, 1, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -425,7 +425,7 @@ void for_jac(
 
             case LogOp:
             CPPAD_ASSERT_NARG_NRES(op, 1, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -434,7 +434,7 @@ void for_jac(
 # if CPPAD_USE_CPLUSPLUS_2011
             case Log1pOp:
             CPPAD_ASSERT_NARG_NRES(op, 1, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -443,7 +443,7 @@ void for_jac(
 
             case MulpvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[1]), var_sparsity
             );
             break;
@@ -451,7 +451,7 @@ void for_jac(
 
             case MulvvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_binary_op(
+            sparse::for_jac_binary_op(
                 i_var, arg, var_sparsity
             );
             break;
@@ -465,7 +465,7 @@ void for_jac(
 
             case PowvpOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 3);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -473,7 +473,7 @@ void for_jac(
 
             case PowpvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 3);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[1]), var_sparsity
             );
             break;
@@ -481,7 +481,7 @@ void for_jac(
 
             case PowvvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 3);
-            forward_sparse_jacobian_binary_op(
+            sparse::for_jac_binary_op(
                 i_var, arg, var_sparsity
             );
             break;
@@ -495,7 +495,7 @@ void for_jac(
             case SignOp:
             CPPAD_ASSERT_NARG_NRES(op, 1, 1);
             // derivative is identically zero but dependency is not
-            if( dependency ) forward_sparse_jacobian_unary_op(
+            if( dependency ) sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             else
@@ -506,7 +506,7 @@ void for_jac(
             case SinOp:
             // cos(x), sin(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -515,7 +515,7 @@ void for_jac(
             case SinhOp:
             // cosh(x), sinh(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -523,7 +523,7 @@ void for_jac(
 
             case SqrtOp:
             CPPAD_ASSERT_NARG_NRES(op, 1, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -578,7 +578,7 @@ void for_jac(
 
             case SubvvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_binary_op(
+            sparse::for_jac_binary_op(
                 i_var, arg, var_sparsity
             );
             break;
@@ -586,7 +586,7 @@ void for_jac(
 
             case SubpvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[1]), var_sparsity
             );
             break;
@@ -594,7 +594,7 @@ void for_jac(
 
             case SubvpOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -603,7 +603,7 @@ void for_jac(
             case TanOp:
             // tan(x)^2, tan(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -612,7 +612,7 @@ void for_jac(
             case TanhOp:
             // tanh(x)^2, tanh(x)
             CPPAD_ASSERT_NARG_NRES(op, 1, 2);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -731,7 +731,7 @@ void for_jac(
 
             case ZmulpvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[1]), var_sparsity
             );
             break;
@@ -739,7 +739,7 @@ void for_jac(
 
             case ZmulvpOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_unary_op(
+            sparse::for_jac_unary_op(
                 i_var, size_t(arg[0]), var_sparsity
             );
             break;
@@ -747,7 +747,7 @@ void for_jac(
 
             case ZmulvvOp:
             CPPAD_ASSERT_NARG_NRES(op, 2, 1);
-            forward_sparse_jacobian_binary_op(
+            sparse::for_jac_binary_op(
                 i_var, arg, var_sparsity
             );
             break;
